from datetime import timedelta

import typing_extensions
from django.db import models


# TODO: comment classes


class Signs(models.Model):
    """
    Class represents signs of email
    """
    sign_id = models.AutoField(primary_key=True, unique=True)
    text = models.CharField(max_length=300)


class Types(models.Model):
    """
    Class represents Types of email
    """
    type_id = models.AutoField(primary_key=True, unique=True)
    type = models.CharField(max_length=300)

    def __str__(self):
        return self.type.__str__()

# TODO sign relation ondelete protect
class Emails(models.Model):
    """
    Class represents emails
    """
    email_id = models.AutoField(primary_key=True, unique=True)
    subject = models.TextField()
    sender = models.TextField()
    recipient = models.TextField()
    date = models.DateTimeField()
    cc = models.TextField()
    body = models.TextField()
    type = models.ForeignKey(Types, on_delete=models.PROTECT)
    sign = models.ManyToManyField(Signs, blank=True)
    gameruns = models.ManyToManyField('GameRuns', through='EmailGameRuns', related_name='gameruns')


class GameRuns(models.Model):
    """
    Class represents gameruns
    """
    gamerun_id = models.AutoField(primary_key=True, unique=True)
    player_name = models.CharField(max_length=100)
    emails = models.ManyToManyField(Emails, through='EmailGameRuns', related_name='emails_in_gamerun')
    start_time = models.DateTimeField(auto_now_add=True)
    end_time = models.DateTimeField(null=True)


class EmailGameRuns(models.Model):
    """
    Class represents manyToMany table of emails and gameruns
    """
    email = models.ForeignKey(Emails, on_delete=models.CASCADE)
    game_run = models.ForeignKey(GameRuns, on_delete=models.CASCADE)
    order = models.IntegerField(default=0)
    player_answer = models.BooleanField(null=True)
    duration = models.DurationField(default=timedelta())

    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=order):
        if self.order == 0:
            try:
                recent = EmailGameRuns.objects.filter(
                    game_run__exact=self.game_run
                ).order_by(
                    '-order'
                )[0]
                self.order = recent.order + 1
            except IndexError:
                self.order = 1

        super(EmailGameRuns, self).save(force_insert, force_update)


class Settings(models.Model):
    """
    Class represents settings of the game
    """
    setting_id = models.AutoField(primary_key=True, unique=True)
    item = models.CharField(max_length=100)
    value = models.IntegerField(default=0)
